@@ -20,11 +20,9 @@
 #include <aws/core/utils/FutureOutcome.h>
 #include <aws/core/utils/Outcome.h>
 #include <aws/core/utils/threading/Executor.h>
-#include <aws/core/utils/threading/SameThreadExecutor.h>
 #include <aws/core/http/HttpResponse.h>
 #include <aws/core/http/HttpClientFactory.h>
 #include <smithy/identity/signer/built-in/SignerProperties.h>
-#include <smithy/identity/auth/built-in/SigV4AuthSchemeOption.h>
 #include <smithy/client/AwsLegacyClient.h>
 
 namespace smithy {
@@ -204,9 +202,9 @@ namespace client
                                  false/*retryable*/);
         }
 
-        SigningOutcome SignHttpRequest(std::shared_ptr<HttpRequest> httpRequest, const AuthSchemeOption& targetAuthSchemeOption) const override
+        SigningOutcome SignHttpRequest(std::shared_ptr<HttpRequest> httpRequest, const AwsSmithyClientAsyncRequestContext& ctx) const override
         {
-            return AwsClientRequestSigning<AuthSchemesVariantT>::SignRequest(httpRequest, targetAuthSchemeOption, m_authSchemes);
+            return AwsClientRequestSigning<AuthSchemesVariantT>::SignRequest(httpRequest, ctx, m_authSchemes);
         }
 
         bool AdjustClockSkew(HttpResponseOutcome& outcome, const AuthSchemeOption& authSchemeOption) const override
@@ -214,6 +212,14 @@ namespace client
             return AwsClientRequestSigning<AuthSchemesVariantT>::AdjustClockSkew(outcome, authSchemeOption, m_authSchemes);
         }
 
+        IdentityOutcome ResolveIdentity(const AwsSmithyClientAsyncRequestContext& ctx) const override {
+          return AwsClientRequestSigning<AuthSchemesVariantT>::ResolveIdentity(ctx, m_authSchemes);
+        }
+
+        GetContextEndpointParametersOutcome GetContextEndpointParameters(const AwsSmithyClientAsyncRequestContext& ctx) const override {
+          return GetContextEndpointParametersImpl(ctx);
+        }
+
         ResponseT MakeRequestDeserialize(Aws::AmazonWebServiceRequest const * const request,
                                      const char* requestName,
                                      Aws::Http::HttpMethod method,
@@ -222,7 +228,7 @@ namespace client
             auto httpResponseOutcome = MakeRequestSync(request, requestName, method, std::move(endpointCallback));
             return m_serializer->Deserialize(std::move(httpResponseOutcome), GetServiceClientName(), requestName);
         }
-
+        
         Aws::String GeneratePresignedUrl(
             EndpointUpdateCallback&& endpointCallback,
             Aws::Http::HttpMethod method,
@@ -232,110 +238,66 @@ namespace client
             const Aws::Http::HeaderValueCollection& customizedHeaders,
             const std::shared_ptr<Aws::Http::ServiceSpecificParameters> serviceSpecificParameters) const
         {
-            AwsSmithyClientAsyncRequestContext ctx;
-            auto authSchemeOptionOutcome = SelectAuthSchemeOption( ctx);
-            auto authSchemeOption = std::move(authSchemeOptionOutcome.GetResultWithOwnership());
-            
-            Aws::Endpoint::EndpointParameters epParams = Aws::Endpoint::EndpointParameters();
-            const auto authSchemeEpParams = authSchemeOption.endpointParameters();
-            epParams.insert(epParams.end(), authSchemeEpParams.begin(), authSchemeEpParams.end());
-            if(serviceSpecificParameters)
-            {
-                auto bucketIt = serviceSpecificParameters->parameterMap.find("bucketName");
-                if(bucketIt != serviceSpecificParameters->parameterMap.end())
+            ExtractUriCallback getUriCallback = [&](Aws::Http::URI& uri, Aws::String& signerRegionOverride,
+                Aws::String& signerServiceNameOverride, const AuthSchemeOption& authSchemeOption) -> bool{
+
+                Aws::Endpoint::EndpointParameters epParams = Aws::Endpoint::EndpointParameters();
+                const auto authSchemeEpParams = authSchemeOption.endpointParameters();
+                epParams.insert(epParams.end(), authSchemeEpParams.begin(), authSchemeEpParams.end());
+                if(serviceSpecificParameters)
                 {
-                    auto bucket = bucketIt->second;
-                    epParams.emplace_back(Aws::String("Bucket"), bucket);
+                    auto bucketIt = serviceSpecificParameters->parameterMap.find("bucketName");
+                    if(bucketIt != serviceSpecificParameters->parameterMap.end())
+                    {
+                        auto bucket = bucketIt->second;
+                        epParams.emplace_back(Aws::String("Bucket"), bucket);
+                    }
                 }
-            }
 
-            auto epResolutionOutcome = this->ResolveEndpoint(std::move(epParams), std::move(endpointCallback));
-            if (!epResolutionOutcome.IsSuccess())
-            {
-                AWS_LOGSTREAM_ERROR(ServiceNameT, "Presigned URL generating failed. Encountered error: " << epResolutionOutcome.GetError().GetMessage());
-                return {};
-            }
-            auto endpoint = std::move(epResolutionOutcome.GetResultWithOwnership());
-            const Aws::Http::URI& uri = endpoint.GetURI();
-            auto signerRegionOverride = region;
-            auto signerServiceNameOverride = serviceName;
-            //signer name is needed for some identity resolvers
-            if (endpoint.GetAttributes()) {
-                if (endpoint.GetAttributes()->authScheme.GetSigningRegion()) {
-                    signerRegionOverride = endpoint.GetAttributes()->authScheme.GetSigningRegion()->c_str();
+                auto epResolutionOutcome = this->ResolveEndpoint(std::move(epParams), std::move(endpointCallback));
+                if (!epResolutionOutcome.IsSuccess())
+                {
+                    AWS_LOGSTREAM_ERROR(ServiceNameT, "Presigned URL generating failed. Encountered error: " << epResolutionOutcome.GetError().GetMessage());
+                    return false;
                 }
-                if (endpoint.GetAttributes()->authScheme.GetSigningRegionSet()) {
-                    signerRegionOverride = endpoint.GetAttributes()->authScheme.GetSigningRegionSet()->c_str();
+                auto endpoint = std::move(epResolutionOutcome.GetResultWithOwnership());
+                uri = endpoint.GetURI();
+                signerRegionOverride = region;
+                signerServiceNameOverride = serviceName;
+                //signer name is needed for some identity resolvers
+                if (endpoint.GetAttributes()) {
+                    if (endpoint.GetAttributes()->authScheme.GetSigningRegion()) {
+                        signerRegionOverride = endpoint.GetAttributes()->authScheme.GetSigningRegion()->c_str();
+                    }
+                    if (endpoint.GetAttributes()->authScheme.GetSigningRegionSet()) {
+                        signerRegionOverride = endpoint.GetAttributes()->authScheme.GetSigningRegionSet()->c_str();
+                    }
+                    if (endpoint.GetAttributes()->authScheme.GetSigningName()) {
+                        signerServiceNameOverride = endpoint.GetAttributes()->authScheme.GetSigningName()->c_str();
+                    }
                 }
-                if (endpoint.GetAttributes()->authScheme.GetSigningName()) {
-                    signerServiceNameOverride = endpoint.GetAttributes()->authScheme.GetSigningName()->c_str();
+                return true;
+            };
+
+            CreateHttpRequestCallback createHttpRequestCallback = [&customizedHeaders, &serviceSpecificParameters](const Aws::Http::URI& uri, const Aws::Http::HttpMethod& method) -> std::shared_ptr<HttpRequest> {
+                std::shared_ptr<HttpRequest> request = CreateHttpRequest(uri, method, Aws::Utils::Stream::DefaultResponseStreamFactoryMethod);
+                request->SetServiceSpecificParameters(serviceSpecificParameters);
+                for (const auto& it: customizedHeaders)
+                {
+                    request->SetHeaderValue(it.first.c_str(), it.second);
                 }
-            }
-            std::shared_ptr<HttpRequest> request = CreateHttpRequest(uri, method, Aws::Utils::Stream::DefaultResponseStreamFactoryMethod);
-            request->SetServiceSpecificParameters(serviceSpecificParameters);
-            for (const auto& it: customizedHeaders)
-            {
-                request->SetHeaderValue(it.first.c_str(), it.second);
-            }
-            if (AwsClientRequestSigning<AuthSchemesVariantT>::PreSignRequest(request, authSchemeOption, m_authSchemes, signerRegionOverride, signerServiceNameOverride, expirationInSeconds).IsSuccess())
-            {
-                return request->GetURIString();
-            }
-            return {};
+                return request;
+            };
+
+            return GeneratePresignedUrl(
+                std::move(getUriCallback),
+                std::move(createHttpRequestCallback),
+                method,
+                region,
+                serviceName,
+                expirationInSeconds);
         }
 
-        //legacy
-        Aws::String GeneratePresignedUrl(const Aws::Http::URI& uri,
-                                                  Aws::Http::HttpMethod method,
-                                                  const Aws::String& region,
-                                                  const Aws::String& serviceName,
-                                                  long long expirationInSeconds,
-                                                  const Aws::Http::HeaderValueCollection& customizedHeaders,
-                                                  const std::shared_ptr<Aws::Http::ServiceSpecificParameters> serviceSpecificParameters) const
-        {
-            std::shared_ptr<HttpRequest> request = CreateHttpRequest(uri, method, Aws::Utils::Stream::DefaultResponseStreamFactoryMethod);
-            request->SetServiceSpecificParameters(serviceSpecificParameters);
-            for (const auto& it: customizedHeaders)
-            {
-                request->SetHeaderValue(it.first.c_str(), it.second);
-            }
-            AwsSmithyClientAsyncRequestContext ctx;
-            auto authSchemeOptionOutcome = SelectAuthSchemeOption( ctx);
-            auto authSchemeOption = std::move(authSchemeOptionOutcome.GetResultWithOwnership());
-            if (AwsClientRequestSigning<AuthSchemesVariantT>::PreSignRequest(request, authSchemeOption, m_authSchemes, region, serviceName, expirationInSeconds).IsSuccess())
-            {
-                return request->GetURIString();
-            }
-            return {};
-        }
-
-        
-        Aws::String GeneratePresignedUrl(const Aws::Endpoint::AWSEndpoint& endpoint,
-                                                  Aws::Http::HttpMethod method,
-                                                  const Aws::String& region,
-                                                  const Aws::String& serviceName,
-                                                  long long expirationInSeconds,
-                                                  const Aws::Http::HeaderValueCollection& customizedHeaders,
-                                                  const std::shared_ptr<Aws::Http::ServiceSpecificParameters> serviceSpecificParameters) const
-        {
-            const Aws::Http::URI& uri = endpoint.GetURI();
-            auto signerRegionOverride = region;
-            auto signerServiceNameOverride = serviceName;
-            // signer name is needed for some identity resolvers
-            if (endpoint.GetAttributes()) {
-                if (endpoint.GetAttributes()->authScheme.GetSigningRegion()) {
-                    signerRegionOverride = endpoint.GetAttributes()->authScheme.GetSigningRegion()->c_str();
-                }
-                if (endpoint.GetAttributes()->authScheme.GetSigningRegionSet()) {
-                    signerRegionOverride = endpoint.GetAttributes()->authScheme.GetSigningRegionSet()->c_str();
-                }
-                if (endpoint.GetAttributes()->authScheme.GetSigningName()) {
-                    signerServiceNameOverride = endpoint.GetAttributes()->authScheme.GetSigningName()->c_str();
-                }
-            }
-            return GeneratePresignedUrl(uri, method, signerRegionOverride, signerServiceNameOverride, expirationInSeconds, customizedHeaders, serviceSpecificParameters);
-        }
-    
         /* Service client specific config, the actual object is stored in AwsSmithyClientBase by pointer
          * In order to avoid config object duplication, smithy template client access it by a reference.
          * So that base client has it by base config pointer, child smithy client has it by child config reference.
@@ -346,6 +308,35 @@ namespace client
         Aws::UnorderedMap<Aws::String, AuthSchemesVariantT> m_authSchemes{};
         std::shared_ptr<SerializerT> m_serializer{};
     private:
+        using ExtractUriCallback = std::function<bool (Aws::Http::URI&, Aws::String& region, Aws::String& serviceName,const AuthSchemeOption&)>;
+        using CreateHttpRequestCallback = std::function<std::shared_ptr<HttpRequest>  (const Aws::Http::URI&, const Aws::Http::HttpMethod&)>;
+
+        Aws::String GeneratePresignedUrl(
+            ExtractUriCallback&& getUriCallback,
+            CreateHttpRequestCallback&& createHttpRequestCallback,
+            Aws::Http::HttpMethod method,
+            const Aws::String& region,
+            const Aws::String& serviceName,
+            long long expirationInSeconds) const
+        {
+            AwsSmithyClientAsyncRequestContext ctx;
+            auto authSchemeOptionOutcome = SelectAuthSchemeOption( ctx);
+            auto authSchemeOption = std::move(authSchemeOptionOutcome.GetResultWithOwnership());
+            Aws::Http::URI uri;
+            Aws::String signerRegionOverride = region;
+            Aws::String signerServiceNameOverride = serviceName;
+            if(!getUriCallback(uri , signerRegionOverride, signerServiceNameOverride, authSchemeOption))
+            {
+                return {};
+            }
+            std::shared_ptr<HttpRequest> request = createHttpRequestCallback(uri, method);
+            if (AwsClientRequestSigning<AuthSchemesVariantT>::PreSignRequest(request, authSchemeOption, m_authSchemes, signerRegionOverride, signerServiceNameOverride, expirationInSeconds).IsSuccess())
+            {
+                return request->GetURIString();
+            }
+            return {};
+        }
+
         friend class AwsLegacyClientT<ServiceNameT, ResponseT, AwsSmithyClientT<ServiceNameT, 
             ServiceClientConfigurationT,
             ServiceAuthSchemeResolverT,
@@ -354,6 +345,21 @@ namespace client
             SerializerT,
             ResponseT,
             ErrorMarshallerT>>;
+
+        GetContextEndpointParametersOutcome GetContextEndpointParametersImpl(const AwsSmithyClientAsyncRequestContext& ctx) const {
+          Aws::Vector<Aws::Endpoint::EndpointParameter> endpointParameters;
+          const auto resolvedAccountId = ctx.m_awsIdentity->accountId();
+          const auto resolvedNonEmptyAccountId = resolvedAccountId.has_value() && !resolvedAccountId.value().empty();
+          // Set user agent if account ID was resolved in identity provider
+          if (resolvedNonEmptyAccountId) {
+            ctx.m_pRequest->AddUserAgentFeature(Aws::Client::UserAgentFeature::RESOLVED_ACCOUNT_ID);
+          }
+          // Only set EP param if client configuration does not have a configured account ID and we resolved a account id
+          if (resolvedNonEmptyAccountId && m_clientConfiguration.accountId.empty()) {
+            endpointParameters.emplace_back("AccountId", resolvedAccountId.value(), Aws::Endpoint::EndpointParameter::ParameterOrigin::OPERATION_CONTEXT);
+          }
+          return endpointParameters;
+        }
     };
 
 } // namespace client
